@@ -24,6 +24,7 @@ import numpy as np
 from pandas import DataFrame
 from svo_filters import Filter
 
+
 from . import mcmc as mc
 from . import utilities as u
 
@@ -95,7 +96,7 @@ class Spectrum:
             raise TypeError("Wavelength array must be in astropy.units.quantity.Quantity length units, e.g. 'um'")
 
         # Check flux units are flux density
-        if not u.equivalent(flux, u.FLAM):
+        if not u.equivalent(flux, (u.FLAM, q.Jy)):
             raise TypeError("Flux array must be in astropy.units.quantity.Quantity flux density units, e.g. 'erg/s/cm2/A'")
 
         # Generate uncertainty array
@@ -104,7 +105,7 @@ class Spectrum:
 
         # Make sure the uncertainty array is in the correct units
         if unc is not None:
-            if not u.equivalent(unc, u.FLAM):
+            if not u.equivalent(unc, (u.FLAM, q.Jy)):
                 raise TypeError("Uncertainty array must be in astropy.units.quantity.Quantity flux density units, e.g. 'erg/s/cm2/A'")
 
         # Replace negatives, zeros, and infs with nans
@@ -548,25 +549,44 @@ class Spectrum:
             The astropy units of the SED wavelength
         """
         # Check the units
-        if not u.equivalent(flux_units, u.FLAM):
-            raise TypeError("flux_units must be in flux density units, e.g. 'erg/s/cm2/A'")
+        if not u.equivalent(flux_units, (u.FLAM, q.Jy)):
+            raise TypeError("flux_units must be in flux density units, e.g. 'erg/s/cm2/A' or 'Jy'")
 
-        # Update the flux and unc arrays
-        self._flux = self._flux * self.flux_units.to(flux_units)
-        if self.unc is not None:
-            self._unc = self._unc * self.flux_units.to(flux_units)
+        # Check if units are Fnu of Flam
+        if u.equivalent(flux_units, q.Jy) and u.equivalent(self.flux_units, u.FLAM):
+
+            # Convert native FLAM units to Jy
+            self._flux = self._flux * self.wave ** 2 * 3.34e-19
+            if self.unc is not None:
+                self._unc = self._unc * self.wave ** 2 * 3.34e-19
+
+        elif u.equivalent(self.flux_units, q.Jy) and u.equivalent(flux_units, u.FLAM):
+
+            # Convert native Jy units to FLAM
+            self._flux = self._flux * 3e18 / (self.wave ** 2)
+            if self.unc is not None:
+                self._unc = self._unc * 3e18 / (self.wave ** 2)
+
+        else:
+
+            # Update the flux and unc arrays
+            self._flux = self._flux * self.flux_units.to(flux_units)
+            if self.unc is not None:
+                self._unc = self._unc * self.flux_units.to(flux_units)
 
         # Set the flux_units
         self._flux_units = flux_units
         self._set_units()
 
-    def integrate(self, units=q.erg / q.s / q.cm**2):
+    def integrate(self, units=q.erg / q.s / q.cm**2, n_samples=10):
         """Calculate the area under the spectrum
 
         Parameters
         ----------
         units: astropy.units.quantity.Quantity
             The target units for the integral
+        n_samples: int
+            The number of samples to take for uncertainty calculations
 
         Returns
         -------
@@ -585,11 +605,23 @@ class Spectrum:
         val = (np.trapz(spec[1], x=spec[0]) * m).to(units)
 
         if self.unc is None:
-            unc = None
+            vunc = None
         else:
-            unc = np.sqrt(np.nansum((spec[2] * np.gradient(spec[0]) * m)**2)).to(units)
 
-        return val, unc
+            # Sum approximation to integral
+            # vunc = np.sqrt(np.nansum((spec[2] * np.gradient(spec[0]) * m)**2)).to(units)
+
+            # Bootstrap errors
+            uvals = []
+            for n in range(n_samples):
+                usamp = np.random.normal(spec[1], spec[2])
+                err = (np.trapz(usamp, x=spec[0]) * m).to(units)
+                uvals.append(abs(err.value - val.value))
+
+            # Get 1-sigma of distribution
+            vunc = np.max(abs(np.asarray(uvals))) * units
+
+        return val, vunc
 
     @copy_raw
     def interpolate(self, wave):
