@@ -14,8 +14,6 @@ import warnings
 
 import astropy.constants as ac
 from astropy.io import fits, ascii
-from astropy.modeling.blackbody import blackbody_lambda
-from astropy.modeling import models
 import astropy.table as at
 import astropy.units as q
 import bokeh.palettes as bpal
@@ -28,6 +26,7 @@ warnings.simplefilter('ignore')
 
 # Valid dtypes for units
 UNITS = q.core.PrefixUnit, q.core.Unit, q.core.CompositeUnit, q.quantity.Quantity, q.core.IrreducibleUnit
+FLAM = q.erg / q.s / q.cm**2 / q.AA
 
 # A dict of BDNYCdb band names to work with sedkit
 PHOT_ALIASES = {'2MASS_J': '2MASS.J', '2MASS_H': '2MASS.H',
@@ -48,30 +47,6 @@ PHOT_ALIASES = {'2MASS_J': '2MASS.J', '2MASS_H': '2MASS.H',
                 'FourStar_J3': 'FourStar.J3', 'HST_F125W': 'WFC3_IR.F125W'}
 
 
-@models.custom_model
-def blackbody(wavelength, temperature=2000):
-    """
-    Generate a blackbody of the given temperature at the given wavelengths
-
-    Parameters
-    ----------
-    wavelength: array-like
-        The wavelength array [um]
-    temperature: float
-        The temperature of the star [K]
-
-    Returns
-    -------
-    astropy.quantity.Quantity
-        The blackbody curve
-    """
-    wavelength = q.Quantity(wavelength, "um")
-    temperature = q.Quantity(temperature, "K")
-    max_val = blackbody_lambda((ac.b_wien / temperature).to(q.um), temperature).value
-
-    return blackbody_lambda(wavelength, temperature).value / max_val
-
-
 def color_gen(colormap='viridis', key=None, n=10):
     """Color generator for Bokeh plots
 
@@ -131,7 +106,8 @@ def convert_mag(band, mag, mag_unc, old='AB', new='Vega'):
     # TODO: Add other bandpasses
     AB_to_Vega = {'Johnson.U': 0.79, 'Johnson.B': -0.09, 'Johnson.V': 0.02, 'Cousins.R': 0.21, 'Cousins.I': 0.45,
                   '2MASS.J': 0.91, '2MASS.H': 1.39, '2MASS.Ks': 1.85,
-                  'SDSS.u': 0.91, 'SDSS.g': -0.08, 'SDSS.r': 0.16, 'SDSS.i': 0.37, 'SDSS.z': 0.54}
+                  'SDSS.u': 0.91 - 0.04, 'SDSS.g': -0.08, 'SDSS.r': 0.16, 'SDSS.i': 0.37, 'SDSS.z': 0.54 + 0.02,
+                  'PS1.g': -0.08, 'PS1.r': 0.16, 'PS1.i': 0.37, 'PS1.z': 0.54, 'PS1.y': 0.63}
 
     if old == 'AB' and new == 'Vega':
         corr = AB_to_Vega.get(band, 0)
@@ -202,6 +178,8 @@ def isnumber(s, nan=False):
     ----------
     s: str, int, float
         The input to test
+    nan: bool
+        Consider NaN a number
 
     Returns
     -------
@@ -211,7 +189,7 @@ def isnumber(s, nan=False):
     if isinstance(s, (str, bytes)):
         return s.replace('.', '').replace('-', '').replace(' + ', '').isnumeric()
 
-    elif isinstance(s, (int, float, np.int64, np.float32, np.float64)):
+    elif isinstance(s, (int, float, np.int64, np.float16, np.float32, np.float64)):
         if np.isnan(s) and not nan:
             return False
         else:
@@ -307,7 +285,7 @@ def filter_table(table, **kwargs):
             if not value.endswith('*'):
                 value = value + '$'
 
-            # Strip souble quotes
+            # Strip double quotes
             value = value.replace("'", '').replace('"', '').replace('*', '(.*)')
 
             # Regex
@@ -329,7 +307,7 @@ def filter_table(table, **kwargs):
                 if any([value.startswith(o) for o in ['<', '>', '=']]):
                     value = [value]
 
-                # Assume eqality if no operator
+                # Assume equality if no operator
                 else:
                     value = ['== ' + value]
 
@@ -342,28 +320,53 @@ def filter_table(table, **kwargs):
 
                 # Equality
                 if cond.startswith('='):
-                    v = cond.replace('=', '')
-                    table = table[table[param] == eval(v)]
+                    val = cond.replace('=', '')
+                    idx = []
+                    for i, v in enumerate(table[param].value):
+                        if v is not None:
+                            if v == float(val):
+                                idx.append(i)
+                    table = table[idx]
 
                 # Less than or equal
                 elif cond.startswith('<='):
-                    v = cond.replace('<=', '')
-                    table = table[table[param] <= eval(v)]
+                    val = cond.replace('<=', '')
+                    idx = []
+                    for i, v in enumerate(table[param].value):
+                        if v is not None:
+                            if v <= float(val):
+                                idx.append(i)
+                    table = table[idx]
 
                 # Less than
                 elif cond.startswith('<'):
-                    v = cond.replace('<', '')
-                    table = table[table[param] < eval(v)]
+                    val = cond.replace('<', '')
+                    idx = []
+                    for i, v in enumerate(table[param].value):
+                        if v is not None:
+                            if v < float(val):
+                                idx.append(i)
+                    table = table[idx]
 
                 # Greater than or equal
                 elif cond.startswith('>='):
-                    v = cond.replace('>=', '')
-                    table = table[table[param] >= eval(v)]
+                    val = cond.replace('>=', '')
+                    idx = []
+                    for i, v in enumerate(table[param].value):
+                        if v is not None:
+                            if v >= float(val):
+                                idx.append(i)
+                    table = table[idx]
 
                 # Greater than
                 elif cond.startswith('>'):
-                    v = cond.replace('>', '')
-                    table = table[table[param] > eval(v)]
+                    val = cond.replace('>', '')
+                    idx = []
+                    for i, v in enumerate(table[param].value):
+                        if v is not None:
+                            if v > float(val):
+                                idx.append(i)
+                    table = table[idx]
 
                 else:
                     raise ValueError("'{}' operator not understood.".format(cond))
@@ -523,29 +526,48 @@ def minimize_norm(arr1, arr2, **kwargs):
     return norm_factor
 
 
-def errorbars(fig, x, y, xerr=None, xupper=None, xlower=None, yerr=None, yupper=None, ylower=None, color='red', point_kwargs={}, error_kwargs={}):
+def errorbars(fig, x, y, xerr=None, xupper=None, xlower=None, yerr=None, yupper=None, ylower=None, source=None, color='red', name='errors', **kwargs):
     """
     Hack to make errorbar plots in bokeh
 
     Parameters
     ----------
-    x: sequence
-        The x axis data
-    y: sequence
-        The y axis data
-    xerr: sequence (optional)
-        The x axis errors
-    yerr: sequence (optional)
-        The y axis errors
+    x: sequence, str
+        The x axis data or ColumnDataSource key
+    y: sequence, str
+        The y axis data or ColumnDataSource key
+    xerr: sequence, str (optional)
+        The x axis symmetric errors or ColumnDataSource key
+    xlower: sequence, str (optional)
+        The x axis lower errors or ColumnDataSource key
+    xupper: sequence, str (optional)
+        The x axis upper errors or ColumnDataSource key
+    yerr: sequence, str (optional)
+        The y axis symmetric errors or ColumnDataSource key
+    ylower: sequence, str (optional)
+        The y axis lower errors or ColumnDataSource key
+    yupper: sequence, str (optional)
+        The y axis upper errors or ColumnDataSource key
     color: str
         The marker and error bar color
-    point_kwargs: dict
-        kwargs for the point styling
-    error_kwargs: dict
-        kwargs for the error bar styling
+    name: str
+        A name for the glyph
     legend: str
         The text for the legend
     """
+    if source is not None:
+
+        # Get data from ColumnDataSource
+        data = source.data
+        x = data[x]
+        xerr = data.get(xerr)
+        xlower = data.get(xlower)
+        xupper = data.get(xupper)
+        y = data[y]
+        yerr = data.get(yerr)
+        ylower = data.get(ylower)
+        yupper = data.get(yupper)
+
     # Add x errorbars if possible
     if xerr is not None or (xupper is not None and xlower is not None):
         x_err_x = []
@@ -553,23 +575,25 @@ def errorbars(fig, x, y, xerr=None, xupper=None, xlower=None, yerr=None, yupper=
 
         # Symmetric uncertainties
         if xerr is not None:
+            if hasattr(xerr, 'unit'):
+                xerr = xerr.value
             for px, py, err in zip(x, y, xerr):
-                try:
-                    x_err_x.append((px - err, px + err))
-                    x_err_y.append((py, py))
-                except TypeError:
-                    pass
+                if isnumber(px) and isnumber(py):
+                    x_err_x.append([px - err, px + err])
+                    x_err_y.append([py, py])
 
         # Asymmetric uncertainties
         elif xupper is not None and xlower is not None:
+            if hasattr(xlower, 'unit'):
+                xlower = xlower.value
+            if hasattr(xupper, 'unit'):
+                xupper = xupper.value
             for px, py, lower, upper in zip(x, y, xlower, xupper):
-                try:
-                    x_err_x.append((px - lower, px + upper))
-                    x_err_y.append((py, py))
-                except TypeError:
-                    pass
+                if isnumber(px) and isnumber(py):
+                    x_err_x.append([px - lower, px + upper])
+                    x_err_y.append([py, py])
 
-        fig.multi_line(x_err_x, x_err_y, color=color, **error_kwargs)
+        fig.multi_line(x_err_x, x_err_y, color=color, name=name, **kwargs)
 
     # Add y errorbars if possible
     if yerr is not None or (yupper is not None and ylower is not None):
@@ -578,23 +602,27 @@ def errorbars(fig, x, y, xerr=None, xupper=None, xlower=None, yerr=None, yupper=
 
         # Symmetric uncertainties
         if yerr is not None:
+            if hasattr(yerr, 'unit'):
+                yerr = yerr.value
             for px, py, err in zip(x, y, yerr):
-                try:
-                    y_err_y.append((py - err, py + err))
-                    y_err_x.append((px, px))
-                except TypeError:
-                    pass
+                if isnumber(px) and isnumber(py):
+                    y_err_y.append([py - err, py + err])
+                    y_err_x.append([px, px])
 
         # Asymmetric uncertainties
         elif yupper is not None and ylower is not None:
+            if hasattr(ylower, 'unit'):
+                ylower = ylower.value
+            if hasattr(yupper, 'unit'):
+                yupper = yupper.value
             for px, py, lower, upper in zip(x, y, ylower, yupper):
-                try:
-                    y_err_y.append((py - lower, py + upper))
-                    y_err_x.append((px, px))
-                except TypeError:
-                    pass
+                if isnumber(px) and isnumber(py):
+                    y_err_y.append([py - lower, py + upper])
+                    y_err_x.append([px, px])
 
-        fig.multi_line(y_err_x, y_err_y, color=color, **error_kwargs)
+        fig.multi_line(y_err_x, y_err_y, color=color, name=name, **kwargs)
+
+    return fig
 
 
 def goodness(f1, f2, e1=None, e2=None, weights=None):
@@ -1002,13 +1030,18 @@ def specType(SpT, types=[i for i in 'OBAFGKMLTY'], verbose=False):
             if isinstance(SpT, bytes):
                 SpT = SpT.decode("utf-8")
 
+            # # Check for multiplicity (i.e. 'M8+L7')
+            # # Use earlier SpT but flag as multiple
+            # if '+' in SpT:
+            #     SpT = SpT.split('+')[0]
+
             # Get the MK spectral class
             MK = types[np.where([i in SpT for i in types])[0][0]]
 
             if MK:
 
                 # Get the stuff before and after the MK class
-                pre, suf = SpT.split(MK)
+                pre, suf = SpT.split(MK)[:2]
 
                 # Get the numerical value
                 val = float(re.findall(r'[0-9]\.?[0-9]?', suf)[0])
@@ -1042,10 +1075,10 @@ def specType(SpT, types=[i for i in 'OBAFGKMLTY'], verbose=False):
 
                 # Check for luminosity class
                 LC = []
-                for cl in ['III', 'V', 'IV']:
+                for cl in ['III', 'II', 'IV', 'I', 'V']:
                     if cl in suf:
+                        suf = suf.replace(cl, '')
                         LC.append(cl)
-                        suf.replace(cl, '')
                 LC = '/'.join(LC) or 'V'
 
                 return [val, unc, pre, grv, LC]
